@@ -1,6 +1,7 @@
 package swift
 
 import (
+	"encoding/json"
 	"fmt"
 	"net/http"
 	"testing"
@@ -212,7 +213,9 @@ func fakeInfrastructureLister(cloudName string) configlisters.InfrastructureList
 	return configlisters.NewInfrastructureLister(fakeIndexer)
 }
 
-func mockConfig(includeStatus bool, endpoint string, secretLister MockSecretNamespaceLister) (driver, imageregistryv1.Config) {
+func mockConfig(
+	includeStatus bool, endpoint string, secretLister MockSecretNamespaceLister, storageManaged bool,
+) (driver, imageregistryv1.Config) {
 	config := imageregistryv1.ImageRegistryConfigStorageSwift{
 		AuthURL:   endpoint,
 		Container: container,
@@ -236,19 +239,116 @@ func mockConfig(includeStatus bool, endpoint string, secretLister MockSecretName
 			},
 		},
 	}
+	if storageManaged {
+		ic.Spec.Storage.ManagementState = imageregistryv1.StorageManagementStateManaged
+	}
 
 	if includeStatus {
 		ic.Status = imageregistryv1.ImageRegistryStatus{
 			Storage: imageregistryv1.ImageRegistryConfigStorage{
 				Swift: &config,
 			},
-			StorageManaged: true,
 		}
 	}
 
 	return d, ic
 }
 
+type handler struct {
+	cur     int
+	headers []int
+}
+
+func (h *handler) request(w http.ResponseWriter, r *http.Request) {
+	defer func() {
+		h.cur++
+	}()
+	w.WriteHeader(h.headers[h.cur])
+}
+
+func (h *handler) setResponses(headers []int) {
+	h.cur = 0
+	h.headers = headers
+}
+
+func TestStorageManagementState(t *testing.T) {
+	th.SetupHTTP()
+	defer th.TeardownHTTP()
+	handleAuthentication(t, "container")
+
+	httpHandler := &handler{}
+	th.Mux.HandleFunc("/", httpHandler.request)
+
+	for _, tt := range []struct {
+		name                    string
+		container               string
+		expectedManagementState string
+		managementState         string
+		headers                 []int
+	}{
+		{
+			name:                    "empty config",
+			expectedManagementState: imageregistryv1.StorageManagementStateManaged,
+		},
+		{
+			name:                    "empty config (management state set)",
+			expectedManagementState: "foo",
+			managementState:         "foo",
+		},
+		{
+			name:                    "container provided (exists)",
+			container:               "a-container",
+			expectedManagementState: imageregistryv1.StorageManagementStateUnmanaged,
+			headers:                 []int{http.StatusOK},
+		},
+		{
+			name:                    "container provided (does not exist)",
+			container:               "another-container",
+			expectedManagementState: imageregistryv1.StorageManagementStateManaged,
+			headers:                 []int{http.StatusNotFound, http.StatusNoContent},
+		},
+		{
+			name:                    "container provided with management set (exists)",
+			container:               "yet-another-container",
+			managementState:         "foo",
+			expectedManagementState: "foo",
+			headers:                 []int{http.StatusOK},
+		},
+		{
+			name:                    "container provided with management set (does not exist)",
+			container:               "container-strikes-back",
+			expectedManagementState: "bar",
+			managementState:         "bar",
+			headers:                 []int{http.StatusNotFound, http.StatusNoContent},
+		},
+	} {
+		t.Run(tt.name, func(t *testing.T) {
+			httpHandler.setResponses([]int{http.StatusNotFound, http.StatusNoContent})
+			if tt.headers != nil {
+				httpHandler.setResponses(tt.headers)
+			}
+
+			drv, installConfig := mockConfig(
+				false, th.Endpoint()+"v3", MockUPISecretNamespaceLister{}, false,
+			)
+
+			installConfig.Spec.Storage.Swift.Container = tt.container
+			if tt.managementState != "" {
+				installConfig.Spec.Storage.ManagementState = tt.managementState
+			}
+
+			err := drv.CreateStorage(&installConfig)
+			th.AssertNoErr(t, err)
+
+			th.AssertEquals(
+				t,
+				tt.expectedManagementState,
+				installConfig.Spec.Storage.ManagementState,
+			)
+		})
+	}
+}
+
 func TestSwiftCreateStorageNativeSecret(t *testing.T) {
 	th.SetupHTTP()
 	defer th.TeardownHTTP()
@@ -277,29 +377,124 @@ func TestSwiftCreateStorageNativeSecret(t *testing.T) {
 		}
 	})
 
-	d, installConfig := mockConfig(false, th.Endpoint()+"v3", MockUPISecretNamespaceLister{})
+	d, installConfig := mockConfig(false, th.Endpoint()+"v3", MockUPISecretNamespaceLister{}, false)
 
 	err := d.CreateStorage(&installConfig)
 
 	th.AssertNoErr(t, err)
-	th.AssertEquals(t, true, installConfig.Status.StorageManaged)
+	th.AssertEquals(t, imageregistryv1.StorageManagementStateManaged, installConfig.Spec.Storage.ManagementState)
 	th.AssertEquals(t, "StorageExists", installConfig.Status.Conditions[0].Type)
 	th.AssertEquals(t, operatorapi.ConditionTrue, installConfig.Status.Conditions[0].Status)
 	th.AssertEquals(t, container, installConfig.Status.Storage.Swift.Container)
 }
 
+func TestSwiftRemoveStorageWithContent(t *testing.T) {
+	th.SetupHTTP()
+	defer th.TeardownHTTP()
+	handleAuthentication(t, "container")
+
+	var containerContentListed bool
+	var containerContentDeleted bool
+	var containerDeleted bool
+	th.Mux.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) {
+		switch r.Method {
+		case http.MethodGet:
+			objects := []string{"obj0", "obj1"}
+			if containerContentListed {
+				objects = []string{}
+			}
+			containerContentListed = true
+			w.WriteHeader(http.StatusOK)
+			_ = json.NewEncoder(w).Encode(objects)
+		case http.MethodPost:
+			if containerContentDeleted {
+				w.WriteHeader(http.StatusNotFound)
+				return
+			}
+			containerContentDeleted = true
+			w.WriteHeader(http.StatusOK)
+			_, _ = w.Write([]byte("{}"))
+		case http.MethodDelete:
+			w.WriteHeader(http.StatusNoContent)
+			containerDeleted = true
+		}
+	})
+
+	d, installConfig := mockConfig(true, th.Endpoint()+"v3", MockUPISecretNamespaceLister{}, true)
+
+	_, err := d.RemoveStorage(&installConfig)
+
+	th.AssertNoErr(t, err)
+	th.AssertEquals(t, "StorageExists", installConfig.Status.Conditions[0].Type)
+	th.AssertEquals(t, operatorapi.ConditionFalse, installConfig.Status.Conditions[0].Status)
+	th.AssertEquals(t, "Swift Container Deleted", installConfig.Status.Conditions[0].Reason)
+	th.AssertEquals(t, "", installConfig.Status.Storage.Swift.Container)
+	th.AssertEquals(t, true, containerContentListed)
+	th.AssertEquals(t, true, containerContentDeleted)
+	th.AssertEquals(t, true, containerDeleted)
+}
+
+func TestSwiftRemoveStorageWithContentFailure(t *testing.T) {
+	th.SetupHTTP()
+	defer th.TeardownHTTP()
+	handleAuthentication(t, "container")
+
+	var containerContentListed bool
+	var containerContentDeleted bool
+	var containerDeleted bool
+	th.Mux.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) {
+		switch r.Method {
+		case http.MethodGet:
+			objects := []string{"obj0", "obj1"}
+			if containerContentListed {
+				objects = []string{}
+			}
+			containerContentListed = true
+			w.WriteHeader(http.StatusOK)
+			_ = json.NewEncoder(w).Encode(objects)
+		case http.MethodPost:
+			if containerContentDeleted {
+				w.WriteHeader(http.StatusNotFound)
+				return
+			}
+			containerContentDeleted = false
+			w.WriteHeader(http.StatusOK)
+			respBody := `{"Number Not Found": 0, "Response Status": "200 OK", "Errors": [["obj0", "Internal Error"]], "Number Deleted": 1, "Response Body": ""}`
+			_, _ = w.Write([]byte(respBody))
+		case http.MethodDelete:
+			w.WriteHeader(http.StatusNoContent)
+			containerDeleted = true
+		}
+	})
+
+	d, installConfig := mockConfig(true, th.Endpoint()+"v3", MockUPISecretNamespaceLister{}, true)
+
+	_, err := d.RemoveStorage(&installConfig)
+
+	th.AssertErr(t, err)
+	th.AssertEquals(t, "errors occured during bulk deleting of container registry objects: cannot delete object obj0: Internal Error", err.Error())
+	th.AssertEquals(t, "registry", installConfig.Status.Storage.Swift.Container)
+	th.AssertEquals(t, true, containerContentListed)
+	th.AssertEquals(t, false, containerContentDeleted)
+	th.AssertEquals(t, false, containerDeleted)
+}
+
 func TestSwiftRemoveStorageNativeSecret(t *testing.T) {
 	th.SetupHTTP()
 	defer th.TeardownHTTP()
 	handleAuthentication(t, "container")
 
 	th.Mux.HandleFunc("/"+container, func(w http.ResponseWriter, r *http.Request) {
+		if r.Method == "GET" {
+			w.WriteHeader(http.StatusNotFound)
+			return
+		}
 		th.TestMethod(t, r, "DELETE")
 		th.TestHeader(t, r, "Accept", "application/json")
 		w.WriteHeader(http.StatusNoContent)
 	})
 
-	d, installConfig := mockConfig(true, th.Endpoint()+"v3", MockUPISecretNamespaceLister{})
+	d, installConfig := mockConfig(true, th.Endpoint()+"v3", MockUPISecretNamespaceLister{}, true)
 
 	_, err := d.RemoveStorage(&installConfig)
 
@@ -330,7 +525,7 @@ func TestSwiftStorageExistsNativeSecret(t *testing.T) {
 		w.WriteHeader(http.StatusNoContent)
 	})
 
-	d, installConfig := mockConfig(false, th.Endpoint()+"v3", MockUPISecretNamespaceLister{})
+	d, installConfig := mockConfig(false, th.Endpoint()+"v3", MockUPISecretNamespaceLister{}, false)
 
 	res, err := d.StorageExists(&installConfig)
 
@@ -435,12 +630,12 @@ func TestSwiftCreateStorageCloudConfig(t *testing.T) {
 		}
 	})
 
-	d, installConfig := mockConfig(false, th.Endpoint()+"v3", MockIPISecretNamespaceLister{})
+	d, installConfig := mockConfig(false, th.Endpoint()+"v3", MockIPISecretNamespaceLister{}, false)
 
 	err := d.CreateStorage(&installConfig)
 
 	th.AssertNoErr(t, err)
-	th.AssertEquals(t, true, installConfig.Status.StorageManaged)
+	th.AssertEquals(t, imageregistryv1.StorageManagementStateManaged, installConfig.Spec.Storage.ManagementState)
 	th.AssertEquals(t, "StorageExists", installConfig.Status.Conditions[0].Type)
 	th.AssertEquals(t, operatorapi.ConditionTrue, installConfig.Status.Conditions[0].Status)
 	th.AssertEquals(t, container, installConfig.Status.Storage.Swift.Container)
@@ -466,12 +661,16 @@ func TestSwiftRemoveStorageCloudConfig(t *testing.T) {
 	}
 
 	th.Mux.HandleFunc("/"+container, func(w http.ResponseWriter, r *http.Request) {
+		if r.Method == "GET" {
+			w.WriteHeader(http.StatusNotFound)
+			return
+		}
 		th.TestMethod(t, r, "DELETE")
 		th.TestHeader(t, r, "Accept", "application/json")
 		w.WriteHeader(http.StatusNoContent)
 	})
 
-	d, installConfig := mockConfig(true, th.Endpoint()+"v3", MockIPISecretNamespaceLister{})
+	d, installConfig := mockConfig(true, th.Endpoint()+"v3", MockIPISecretNamespaceLister{}, true)
 
 	_, err := d.RemoveStorage(&installConfig)
 
@@ -516,7 +715,7 @@ func TestSwiftStorageExistsCloudConfig(t *testing.T) {
 		w.WriteHeader(http.StatusNoContent)
 	})
 
-	d, installConfig := mockConfig(false, th.Endpoint()+"v3", MockIPISecretNamespaceLister{})
+	d, installConfig := mockConfig(false, th.Endpoint()+"v3", MockIPISecretNamespaceLister{}, false)
 
 	res, err := d.StorageExists(&installConfig)
 
@@ -539,7 +738,7 @@ func TestSwiftConfigEnvCloudConfig(t *testing.T) {
 		cloudSecretKey: fakeCloudsYAMLData,
 	}
 
-	d, _ := mockConfig(false, "http://localhost:5000/v3", MockIPISecretNamespaceLister{})
+	d, _ := mockConfig(false, "http://localhost:5000/v3", MockIPISecretNamespaceLister{}, false)
 
 	res, err := d.ConfigEnv()
 
@@ -650,7 +849,7 @@ func TestSwiftEndpointTypeObjectStore(t *testing.T) {
 		w.WriteHeader(http.StatusNoContent)
 	})
 
-	d, installConfig := mockConfig(false, th.Endpoint()+"v3", MockUPISecretNamespaceLister{})
+	d, installConfig := mockConfig(false, th.Endpoint()+"v3", MockUPISecretNamespaceLister{}, false)
 
 	res, err := d.StorageExists(&installConfig)
 
@@ -734,7 +933,7 @@ func TestSwiftIsNotAvailable(t *testing.T) {
 		w.WriteHeader(http.StatusNoContent)
 	})
 
-	d, _ := mockConfig(false, th.Endpoint()+"v3", MockUPISecretNamespaceLister{})
+	d, _ := mockConfig(false, th.Endpoint()+"v3", MockUPISecretNamespaceLister{}, false)
 
 	_, err := d.getSwiftClient()
 	// if Swift endpoint is not registered, getSwiftClient should return *ErrEndpointNotFound
@@ -780,7 +979,7 @@ func TestNoPermissionsKeystone(t *testing.T) {
 		w.WriteHeader(http.StatusForbidden)
 	})
 
-	d, _ := mockConfig(false, th.Endpoint()+"v3", MockUPISecretNamespaceLister{})
+	d, _ := mockConfig(false, th.Endpoint()+"v3", MockUPISecretNamespaceLister{}, false)
 
 	conn, err := d.getSwiftClient()
 	th.AssertNoErr(t, err)
@@ -830,7 +1029,7 @@ func TestNoPermissionsSwauth(t *testing.T) {
 		w.WriteHeader(http.StatusUnauthorized)
 	})
 
-	d, _ := mockConfig(false, th.Endpoint()+"v3", MockUPISecretNamespaceLister{})
+	d, _ := mockConfig(false, th.Endpoint()+"v3", MockUPISecretNamespaceLister{}, false)
 
 	conn, err := d.getSwiftClient()
 	th.AssertNoErr(t, err)
